package io.xjar.jar;

import java.util.Iterator;
import java.util.jar.Attributes;
import java.util.Set;
import java.io.Closeable;
import io.xjar.XInjector;
import io.xjar.XKit;
import java.util.jar.Manifest;
import org.apache.commons.compress.archivers.ArchiveEntry;
import io.xjar.XUnclosedOutputStream;
import io.xjar.XUnclosedInputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveOutputStream;
import org.apache.commons.compress.archivers.jar.JarArchiveInputStream;
import java.util.LinkedHashSet;
import java.io.IOException;
import io.xjar.XGo;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.File;
import io.xjar.key.XKey;
import io.xjar.XEntryFilter;
import io.xjar.XConstants;
import io.xjar.XEncryptor;
import org.apache.commons.compress.archivers.jar.JarArchiveEntry;
import io.xjar.XEntryEncryptor;

public class XJarEncryptor extends XEntryEncryptor<JarArchiveEntry> implements XEncryptor, XConstants
{
    private final int level;
    
    public XJarEncryptor(final XEncryptor xEncryptor) {
        this(xEncryptor, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XJarEncryptor(final XEncryptor xEncryptor, final XEntryFilter<JarArchiveEntry> filter) {
        this(xEncryptor, -1, filter);
    }
    
    public XJarEncryptor(final XEncryptor xEncryptor, final int level) {
        this(xEncryptor, level, (XEntryFilter<JarArchiveEntry>)new XJarAllEntryFilter());
    }
    
    public XJarEncryptor(final XEncryptor xEncryptor, final int level, final XEntryFilter<JarArchiveEntry> filter) {
        super(xEncryptor, (XEntryFilter)filter);
        this.level = level;
    }
    
    public void encrypt(final XKey key, final File src, final File dest) throws IOException {
        try (FileInputStream fis = new FileInputStream(src);
             FileOutputStream fos = new FileOutputStream(dest)) {
            this.encrypt(key, fis, fos);
            XGo.make(dest, key);
        }
    }
    
    public void encrypt(final XKey key, final InputStream in, final OutputStream out) throws IOException {
        JarArchiveInputStream zis = null;
        JarArchiveOutputStream zos = null;
        final Set<String> indexes = new LinkedHashSet<String>();
        try {
            zis = new JarArchiveInputStream(in);
            zos = new JarArchiveOutputStream(out);
            zos.setLevel(this.level);
            final XUnclosedInputStream nis = new XUnclosedInputStream((InputStream)zis);
            final XUnclosedOutputStream nos = new XUnclosedOutputStream((OutputStream)zos);
            Manifest manifest = null;
            JarArchiveEntry entry;
            while ((entry = zis.getNextJarEntry()) != null) {
                if (!entry.getName().startsWith(XJarEncryptor.XJAR_SRC_DIR) && !entry.getName().endsWith("XJAR-INF/")) {
                    if (entry.getName().endsWith("XJAR-INF/INDEXES.IDX")) {
                        continue;
                    }
                    if (entry.isDirectory()) {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                    }
                    else if (entry.getName().equals("META-INF/MANIFEST.MF")) {
                        manifest = new Manifest((InputStream)nis);
                        final Attributes attributes = manifest.getMainAttributes();
                        final String mainClass = attributes.getValue("Main-Class");
                        if (mainClass != null) {
                            attributes.putValue("Jar-Main-Class", mainClass);
                            attributes.putValue("Main-Class", "io.xjar.jar.XJarLauncher");
                        }
                        final JarArchiveEntry jarArchiveEntry2 = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry2.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry2);
                        manifest.write((OutputStream)nos);
                    }
                    else {
                        final JarArchiveEntry jarArchiveEntry = new JarArchiveEntry(entry.getName());
                        jarArchiveEntry.setTime(entry.getTime());
                        zos.putArchiveEntry((ArchiveEntry)jarArchiveEntry);
                        final boolean filtered = this.filtrate((Object)entry);
                        if (filtered) {
                            indexes.add(entry.getName());
                        }
                        final XEncryptor encryptor = (XEncryptor)(filtered ? this.xEncryptor : this.xNopEncryptor);
                        try (OutputStream eos = encryptor.encrypt(key, (OutputStream)nos)) {
                            XKit.transfer((InputStream)nis, eos);
                        }
                    }
                    zos.closeArchiveEntry();
                }
            }
            if (!indexes.isEmpty()) {
                final JarArchiveEntry xjarInfDir = new JarArchiveEntry("XJAR-INF/");
                xjarInfDir.setTime(System.currentTimeMillis());
                zos.putArchiveEntry((ArchiveEntry)xjarInfDir);
                zos.closeArchiveEntry();
                final JarArchiveEntry xjarInfIdx = new JarArchiveEntry("XJAR-INF/INDEXES.IDX");
                xjarInfIdx.setTime(System.currentTimeMillis());
                zos.putArchiveEntry((ArchiveEntry)xjarInfIdx);
                for (final String index : indexes) {
                    zos.write(index.getBytes());
                    zos.write(XJarEncryptor.CRLF.getBytes());
                }
                zos.closeArchiveEntry();
            }
            final String mainClass2 = (manifest != null && manifest.getMainAttributes() != null) ? manifest.getMainAttributes().getValue("Main-Class") : null;
            if (mainClass2 != null) {
                XInjector.inject(zos);
            }
            zos.finish();
        }
        finally {
            XKit.close((Closeable)zis);
            XKit.close((Closeable)zos);
        }
    }
    
    public boolean predicate(final InputStream jar) throws IOException {
        JarArchiveInputStream zis = null;
        try {
            zis = new JarArchiveInputStream(jar);
            JarArchiveEntry entry;
            while ((entry = zis.getNextJarEntry()) != null) {
                if (!entry.getName().startsWith(XJarEncryptor.XJAR_SRC_DIR) && !entry.getName().endsWith("XJAR-INF/") && !entry.getName().endsWith("XJAR-INF/INDEXES.IDX") && !entry.isDirectory()) {
                    if (entry.getName().equals("META-INF/MANIFEST.MF")) {
                        continue;
                    }
                    if (this.filtrate((Object)entry)) {
                        return true;
                    }
                    continue;
                }
            }
            return false;
        }
        finally {
            XKit.close((Closeable)zis);
        }
    }
}
